;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'files '("tramp"))
(custom-add-loads 'tramp '("tramp" "tramp-ftp" "tramp-smb"))

;;; custom-load.el ends here
